@@ -1,12 +1,32 @@
+// Copyright (C) 2004-2025 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 #include "mupdf/pdf.h"
 
-#include "../fitz/font-imp.h"
-
 static void
-pdf_run_glyph_func(fz_context *ctx, void *doc, void *rdb, fz_buffer *contents, fz_device *dev, fz_matrix ctm, void *gstate, fz_default_colorspaces *default_cs)
+pdf_run_glyph_func(fz_context *ctx, void *doc, void *rdb, fz_buffer *contents, fz_device *dev, fz_matrix ctm, void *gstate, fz_default_colorspaces *default_cs, void *fill_gstate, void *stroke_gstate)
 {
-	pdf_run_glyph(ctx, doc, (pdf_obj *)rdb, contents, dev, ctm, gstate, default_cs);
+	pdf_run_glyph(ctx, doc, (pdf_obj *)rdb, contents, dev, ctm, gstate, default_cs, fill_gstate, stroke_gstate);
 }
 
 static void
@@ -41,7 +61,7 @@ pdf_load_type3_font(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *d
 
 		if (new_max == 0)
 			new_max = 4;
-		doc->type3_fonts = fz_resize_array(ctx, doc->type3_fonts, new_max, sizeof(*doc->type3_fonts));
+		doc->type3_fonts = fz_realloc_array(ctx, doc->type3_fonts, new_max, fz_font*);
 		doc->max_type3_fonts = new_max;
 	}
 
@@ -65,6 +85,10 @@ pdf_load_type3_font(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *d
 
 		fz_set_font_bbox(ctx, font, bbox.x0, bbox.y0, bbox.x1, bbox.y1);
 
+		font->ascender = bbox.y1;
+		font->descender = bbox.y0;
+		font->ascdesc_src = FZ_ASCDESC_FROM_BOUNDS;
+
 		/* Encoding */
 
 		for (i = 0; i < 256; i++)
@@ -97,7 +121,7 @@ pdf_load_type3_font(fz_context *ctx, pdf_document *doc, pdf_obj *rdb, pdf_obj *d
 					item = pdf_array_get(ctx, diff, i);
 					if (pdf_is_int(ctx, item))
 						k = pdf_to_int(ctx, item);
-					if (pdf_is_name(ctx, item) && k >= 0 && k < nelem(estrings))
+					if (pdf_is_name(ctx, item) && k >= 0 && k < (int)nelem(estrings))
 						estrings[k++] = pdf_to_name(ctx, item);
 				}
 			}
@@ -204,9 +228,25 @@ void pdf_load_type3_glyphs(fz_context *ctx, pdf_document *doc, pdf_font_desc *fo
 				fontdesc->size += 0; // TODO: display list size calculation
 			}
 		}
+
+		/* Derive missing font bbox from char bboxes if there are any. */
+		if (fontdesc->font->flags.invalid_bbox && fontdesc->font->bbox_table != NULL)
+		{
+			/* Union all the char bboxes together. */
+			fz_rect bbox = fz_empty_rect;
+			for (i = 0; i < 256; i++)
+			{
+				if (fontdesc->font->t3procs[i])
+					bbox = fz_union_rect(bbox, fontdesc->font->bbox_table[0][i]);
+			}
+			fontdesc->font->bbox = bbox;
+		}
 	}
 	fz_catch(ctx)
 	{
-		fz_warn(ctx, "Type3 glyph load failed: %s", fz_caught_message(ctx));
+		fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
+		fz_rethrow_if(ctx, FZ_ERROR_SYSTEM);
+		fz_report_error(ctx);
+		fz_warn(ctx, "type3 glyph load failed");
 	}
 }
